<?php
//---------------------------------------------------------------------------
//Charactor set is UTF-8
//漢字コードはＵＴＦ－８
//フォントはＭＳ ゴシック
//---------------------------------------------------------------------------
$gSoftName  = "asuna ver.1.0 : Database Front-end software.";
$gCopyRight = "(C)Copyright 2021 Y.ENDOU All right reserved.";
//---------------------------------------------------------------------------
//	ver.1.0		2021/08/01		初期バージョン
//								即値を多用するのは良くない
//---------------------------------------------------------------------------
/* 注意
1)php.ini の標準設定で request_order = "GP" となっていて、クッキーの値が $_REQUESTに入らない
2)開発環境でPHPのセッション機能が動作しなかったのでCSRF対策を行わなかった
3)カラムの照合順序が統一されていない(ex.utf8_unicode_ciの中にutf32_unicode_ciが混じっているなど)と
検索の際にIllegal mix of collationsエラーが出る
@link https://tmtms.hatenablog.com/entry/2013/04/11/mysql_Illegal_mix_of_collations (2021/07確認)
4)Microsoft Edgeはinput の datetime , datetime-localに不完全対応(2020/10)
5)sql_modeにTRADITIONALが入っていると、入力文字数超過の際にエラーが出る。
  「編集者」カラムは、編集ごとに名前を追加してゆく。TRADITIONALモードにしておくと
  文字数が超過してエラーが出てしまうので、文字数を十分取るかプログラムを変更する必要がある。
6)「安全なウェブサイトの作り方」改訂第7版
(@link https://www.ipa.go.jp/security/vuln/websecurity.html (2021/07確認))を参考にしている
*/
//---------------------------------------------------------------------------
require('prog/config.inc');			//編集する必要がある設定ファイル
require('prog/asuna.inc');			//編集してはいけない設定ファイル
$gDsn = sprintf(kDSN, $gDataBase);	//アクセスするデータベース
require('prog/asuna00.inc');		//共通ルーチン
//---------------------------------------------------------------------------
/**
 * コマンド分岐ルーチン
 *
 * @param boolean kUseUserTB : ユーザーテーブルを使うかどうか true:使う
 * @param string $_REQUEST['Cmd'] : コマンド
 * @param string $_COOKIE[kLogin] : ログインクッキー ログインした日時
 */
if (kUseUserTB) {										//ユーザーテーブルを使う
	if (!isset($_COOKIE[kLogin])) {						//ログインしていない
		if (isset($_REQUEST['Cmd'])) {
			switch ($_REQUEST['Cmd']) {					//コマンドがある
				case 'ユーザー認証':					//ユーザー認証（失敗の場合は戻ってこない）
					require('prog/asuna01.inc');
					Login();
					$_REQUEST['Cmd'] = "テーブル表示";	//ログインに成功したので表を表示する
					break;
				case 'パス変更':						//パスフレーズが空の場合、パスフレーズ変更（失敗の場合は戻ってこない）
					require('prog/asuna01.inc');
					$msg = ChangePass();
					LoginScreen($msg);
					exit();
					break;
				case "ログアウト":						//ログアウト後の再読み込み対応
					require('prog/asuna01.inc');
					LoginScreen();
					exit();
				default:
					UnExpected();						//戻ってこない
			}
		} else {
			require('prog/asuna01.inc');				//コマンドがなければログイン画面
			LoginScreen();
			exit();
		}
	}
} else {
	$_COOKIE[kUserID]   = '-';							//ログファイル用ダミーデータ
	$_COOKIE[kSection]  = '-';
	$_COOKIE[kUserName] = '-';
}

SetDefalutParam();										//$_REQUEST , $_COOKIE , 設定から初期設定データを作る

if (isset($_REQUEST['Cmd'])) {
	switch ($_REQUEST['Cmd']) {
		case 'テーブル表示':							//選択されたテーブルの内容を表示する
		case '並び替え':
			require('prog/asuna02.inc');
			ViewTable();
			break;
			// 新規、表示、編集、複製の場合は、別ウインドウを開いて入力・編集を行う
		case '新規':									//行の編集ウインドウを作る
		case '編集':
		case '複製':
			require('prog/asuna03.inc');
			EditRowWin();
			break;
		case '表示':									//個別のデータを別ウインドウで表示する
			require('prog/asuna03.inc');
			ViewRowWin();
			break;
		case '削除':									//削除確認ウインドウを開く
			require('prog/asuna03.inc');
			DelRowWin();
			break;
		case '空にする':								//テーブルを空にする
			require('prog/asuna02.inc');
			TruncateTable();
			break;
		case '行追加':									//行追加
			require('prog/asuna03.inc');
			InsertRow();
			break;
		case '行変更':									//行変更
			require('prog/asuna03.inc');
			UpdateRow();
			break;
		case '行削除':									//行削除
			require('prog/asuna03.inc');
			DeleteRow();
			break;
		case 'CSV出力':									//CSV出力
			require('prog/asuna02.inc');
			CSVOut();
			break;
		case '空にする実行':							//テーブルを空にする実行
			require('prog/asuna02.inc');
			TruncateTableEx();
			break;
		case 'ログアウト':								//ログアウト
			require('prog/asuna01.inc');
			Logout();
			LoginScreen("<div class=\"message msgbox\">ログアウトしました。</div>\n");
			break;
		case 'パス変更画面':							//パスフレーズ変更画面
			require('prog/asuna01.inc');
			ChangePassScreen();
			break;
		case 'パス変更':								//パスフレーズ変更
			require('prog/asuna01.inc');
			$msg = ChangePass();
			LoginScreen($msg);
			break;
		case 'ユーザー認証':							//ユーザー認証（失敗の場合は戻ってこない）
			require('prog/asuna01.inc');
			Login();
			$_REQUEST['Cmd'] = "テーブル表示";			//ログインに成功したので表を表示する
			require('prog/asuna02.inc');
			ViewTable();
			break;
		default:
			UnExpected();								//戻ってこない
	}
} else {												//コマンドがない場合　テーブル表示
	require('prog/asuna02.inc');
	$_REQUEST['Cmd'] = "テーブル表示(コマンドなし)";	//ログ用
	ViewTable();
}
//---------------------------------------------------------------------------
