<?php
//---------------------------------------------------------------------------
//Charactor set is UTF-8
//漢字コードはＵＴＦ－８
//---------------------------------------------------------------------------
//asuna共通ルーチン
//---------------------------------------------------------------------------
/**
 * ログを記録する
 *
 * @param string $pUserID : ユーザーID
 * @param string $pSection : 所属
 * @param string $pName : 氏名
 * @param string $pTable : 編集テーブル
 * @param string $pOpID : 操作ID
 * @param string $pOperation : 操作
 * @param string $pResult : 結果
 *
 * @return integer : ログのID番号  ログを使わない場合は -1
 */
function WriteLog($pUserID, $pSection, $pName, $pTable, $pOpID, $pOperation, $pResult)
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	if (!kUseLogTB) return (-1);

	$dbg = debug_backtrace();	//バックトレースを生成する
	try {
		$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);
		$pdo->beginTransaction();

		//asuna_log_table はオートインクリメントのIDを持っているので、ここだけ「ユーザーID」
		$insert_str = MakeInsertStr(array('ユーザーID', '所属', '氏名', '編集テーブル', '操作ID', '操作', '結果', '関数'));
		$params = array($pUserID, $pSection, $pName, $pTable, $pOpID, $pOperation, $pResult, $dbg[1]['function']);
		$sql = "INSERT INTO `" . kLOG_TABLE . "` " . $insert_str;
		$stmt = $pdo->prepare($sql);
		$stmt->execute($params);
		$id = $pdo->lastInsertId();					//入力したデータのID (ID支出の番号(文字列)) commitの後に書いてはいけない
		$pdo->commit();
	} catch (PDOException $e) {
		$pdo->rollBack();
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$err = htmlspecialchars($err, ENT_QUOTES, 'UTF-8');
		$err = nl2br($err, false);
		printf("<div class=\"error msgbox\">ログを記入することができませんでした。<br>\n管理者に連絡してください。<br>\n%s</div>\n", $err);
		$id = -1;
	}
	return ($id);
}
//---------------------------------------------------------------------------
/**
 * 「ログアウト」と「パスフレーズ変更」ボタンを作る
 *
 * @param string $_COOKIE[kUserID] : ユーザーID
 * @param string $_COOKIE[kUserName] : ユーザー名
 */
function LogoutButton()
{
	if (!kUseUserTB) return;

	$userID   = htmlspecialchars($_COOKIE[kUserID],   ENT_QUOTES, 'UTF-8');
	$userName = htmlspecialchars($_COOKIE[kUserName], ENT_QUOTES, 'UTF-8');
	if (kUseUserTB) {
		$user = "<span style=\"vertical-align: middle;\">ユーザーID：{$userID}　ユーザー名：{$userName}</span>";
	} else {
		$user = "";
	}
	//---------------表示部---------------

	print <<< kLogoutButton
<section class="cLogout">
	{$user}
	<form action="index.php" method="post" style="display:inline">
		<input type="hidden" name="Cmd" value="ログアウト">
		<input type="submit" value="ログアウト">
	</form>
	<form action="index.php" method="post" style="display:inline">
		<input type="hidden" name="Cmd" value="パス変更画面">
		<input type="hidden" name="UserID" value="{$userID}">
		<input type="submit" value="パスフレーズ変更">
	</form>
</section>

kLogoutButton;
}
//---------------------------------------------------------------------------
/**
 * $_REQUEST , $_COOKIE , 設定から初期設定データ($_REQUEST)を作る
 * スーパーグローバル変数に代入している！
 *
 * @param string $_REQUEST['Table'] : 選択しているテーブルの番号
 * @param string $_REQUEST['Page'] : 表示ページ
 * @param string $_REQUEST['SortCol'] : ソートカラムの番号
 * @param string $_REQUEST['SortOrder'] : ソート順(ASC , DESC)
 * @param string $_REQUEST['KeyWords'] :キーワード文字列
 */
function SetDefalutParam()
{
	global $gTables, $gTables2;
	//---------------処理部---------------
	//管理者ならば利用できるテーブルを増やす
	if (isset($_COOKIE[kUserLevel]) && ($_COOKIE[kUserLevel] == '管理者')) {
		$gTables = array_merge($gTables, $gTables2);		//管理者であればログ、ユーザーテーブルの操作が可能
	}

	//テーブル番号
	$_REQUEST['Table']   = CheckSelectIndex('Table', kTable, count($gTables));
	//表示ページ番号
	$_REQUEST['Page']    = CheckSelectIndex('Page', kPage);
	//ソートカラム番号
	$_REQUEST['SortCol'] = CheckSelectIndex('SortCol', kSortCol);

	//ソート順(ASC , DESC)
	$aSortOrder = "DESC";
	if (isset($_COOKIE[kSortOrder]) && ($_COOKIE[kSortOrder] == "ASC")) $aSortOrder = "ASC";
	if (isset($_REQUEST['SortOrder'])) {
		if ($_REQUEST['SortOrder'] == "ASC") $aSortOrder = "ASC";
		else $aSortOrder = "DESC";
	}
	$_REQUEST['SortOrder'] = $aSortOrder;

	//キーワード
	if (empty($_REQUEST['KeyWords'])) $_REQUEST['KeyWords'] = "";

	//次回アクセス時に同じテーブルを表示
	$exp = time() + 60 * 60 * kLastViewExp;					//表示に関するクッキーをセット
	setcookie(kTable,     $_REQUEST['Table'],     $exp);
	setcookie(kSortCol,   $_REQUEST['SortCol'],   $exp);
	setcookie(kSortOrder, $_REQUEST['SortOrder'], $exp);
}
//---------------------------------------------------------------------------
/**
 * SELECTタグで送られてくる配列の添え字を検査する
 *
 * @param string	$pRequestName : $_REQUEST[]に入れる文字
 * @param string	$pCookieName : $_COOKIE[]に入れる文字
 * @param integer	$pUpper : 配列の上限、0なら上限を検査しない
 *
 * @return integer	配列の添え字、デフォルトで0を返す
 */
function CheckSelectIndex($pRequestName, $pCookieName, $pUpper = 0)
{
	if (isset($_REQUEST[$pRequestName])) {
		if (1 == preg_match("/\\D/", $_REQUEST[$pRequestName])) {		//数字以外の文字が含まれている場合、エラーを出して終了
			UnExpected("REQUEST[{$pRequestName}]に数字以外の文字が含まれている");
		}
		$ret = intval($_REQUEST[$pRequestName]);						//intvalは変換に失敗すると 0 を返す
		if ($ret < 0) {
			UnExpected("REQUEST[{$pRequestName}]が負の値");
		}
		if (($pUpper > 0) && ($ret >= $pUpper)) {						//範囲外の番号はエラーを出して終了
			UnExpected("REQUEST[{$pRequestName}]が上限を超えている");
		}
	} else if (isset($_COOKIE[$pCookieName])) {
		if (1 == preg_match("/\\D/", $_COOKIE[$pCookieName])) {
			UnExpected("COOKIE[{$pCookieName}]に数字以外の文字が含まれている");
		}
		$ret = intval($_COOKIE[$pCookieName]);
		if ($ret < 0) {
			UnExpected("COOKIE[{$pCookieName}]が負の値");
		}
		if (($pUpper > 0) && ($ret >= $pUpper)) {
			UnExpected("COOKIE[{$pCookieName}]が上限を超えている");
		}
	} else {
		$ret = 0;
	}

	return ($ret);
}
//---------------------------------------------------------------------------
/**
 * カラム名一覧を得る
 *
 * @param string $_REQUEST['Table'] : 選択しているテーブルの番号
 * @param string[] $pFields : カラム名を入れて返す
 * @param string[] $pColumns : テーブルの構造を入れて返す
 *
 * @return string : プライマリーキーの名前
 */
function GetColumns(&$pFields, &$pColumns)
{
	global $gPDOOption, $gDsn;
	global $gTables;
	//---------------処理部---------------
	$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

	$sql = sprintf("SHOW FULL COLUMNS FROM `%s` ", $gTables[$_REQUEST['Table']]);	//bindValue はカラムの値に使用するものでテーブルには使えない
	$stmt = $pdo->prepare($sql);
	$stmt->execute();

	$pColumns = $stmt->fetchAll(PDO::FETCH_ASSOC);
	$pFields = array();
	$PrimarykeyName = "";
	foreach ($pColumns as $val) {
		$pFields[] = $val['Field'];
		if ($val['Key'] == 'PRI') {
			$PrimarykeyName = $val['Field'];
		}
	}
	return ($PrimarykeyName);
}
//---------------------------------------------------------------------------
/**
 * １次元配列の値をサニタイズして返す
 *
 * @param array	$p : サニタイズする配列
 * @param bool	$br : nl2brも行うか
 * @return array : サニタイズ後の配列
 */
function array_sanitize($p, $br = false)
{
	$ret = array();
	foreach ($p as $key => $val) {
		if ($br) {
			$ret[$key] = nl2br(htmlspecialchars($val, ENT_QUOTES, 'UTF-8'), false);
		} else {
			$ret[$key] = htmlspecialchars($val, ENT_QUOTES, 'UTF-8');
		}
	}
	return ($ret);
}
//---------------------------------------------------------------------------
/**
 * 配列の要素を文字で囲んで返す
 *
 * @param array		$p : 囲まれる配列
 * @param string	$a : 囲む文字列
 * @param string	$b : 囲む文字列
 *
 * @return array	文字に囲まれた配列
 */
function array_enclose($p, $a = '`', $b = '`')
{
	$ret = array();
	foreach ($p as $key => $val) {
		$ret[$key] = $a . $val . $b;
	}
	return ($ret);
}
//---------------------------------------------------------------------------
/**
 * $_REQUESTのデータを元にSELECT文のWHERE句を作る(INSERT,UPDATEのWHERE句ではない)
 *
 * @param array		&$pParams : $stmt->execute()の引数にする配列を入れて返す
 * @param array		$pConf : 絞り込みの条件項目 $_REQUEST['キー']
 * @param bool		$pWhere : 文字列の先頭にWHEREを付けるかどうか。
 *
 * @return string	WHERE句の文字列
 * 					ex. WHERE (`column1` = ?) AND (`column2` = ?) AND (`column3` = ?)
 */
function MakeWherePhraseWithRequest(&$pParams, $pConf, $pWhere = false)
{
	if ($pWhere) $where = "WHERE ";
	else $where = "";

	$pParams = array();
	$where_array = array();

	foreach ($pConf as $val) {
		if (!empty($_REQUEST[$val])) {		//$_REQUEST野中に絞り込みキーの値があればWHERE句にする
			$where_array[] = "(`$val` = ?)";
			$pParams[] = $_REQUEST[$val];
		}
	}
	if (count($where_array) != 0) {
		$where_str = $where . implode(' AND ', $where_array) . " ";
	} else {
		$where_str = "";
	}
	return ($where_str);
}
//---------------------------------------------------------------------------
/**
 * カラム名配列を用いてWHERE句の文字列を作る(INSERT,UPDATEのWHERE句ではない)
 *
 * @param array		$pColumn : カラム名配列
 * @param array		$pParams : バインドするデータ配列－あいまい検索の場合書き換える
 * @param bool		$pWhere : 文字列の先頭にWHEREを付けるかどうか。
 * @param bool		$pLike : LIKE にするか = にするか
 *
 * @return string	WHERE句の文字列
 * ex. "(`colum1` = ?) AND (`colum2` = ?) AND (`colum3` = ?) AND (`colum4` = ?) " が返る
 *
 * 使い方
 * $column = array('colum1','colum2','colum3','colum4');
 * $params = array('param1','param2','param3','param4');
 * $where_str = MakeWherePhraseWithColumn($column,$params);
 * $sql = "SELECT * FROM table WHERE " . $where_str;
 * $stmt = $pdo->prepare($sql);
 * $stmt->execute($params);
 */
function MakeWherePhraseWithColumn($pColumn, &$pParams, $pWhere = false, $pLike = false)
{
	if (count($pColumn) == 0) return ("");

	if ($pLike) {										//あいまい検索の場合 $pParams[] を%で囲んで返す
		$like = "LIKE";
		$pParams = array_enclose($pParams, '%', '%');	//WHERE句ではなくバインドするパラメータのほうに % を付ける
	} else {
		$like = "=";
	}

	$where_array = array();
	foreach ($pColumn as $val) {
		$where_array[] = sprintf("(`%s` %s ?)", $val, $like);
	}

	$where = $pWhere ? "WHERE " : "";
	$where_str = $where . implode(' AND ', $where_array) . " ";

	return ($where_str);
}
//---------------------------------------------------------------------------
/**
 * キーワードを解析してSELECT文のWHERE句を作る(INSERT,UPDATEのWHERE句ではない)
 *
 * @param array		&$pParams : $stmt->execute()の引数にする配列を入れて返す。
 * 								ex. array( [0] => '%キー１%' [1] => '%キー２%' [2] => '%キー３%' )
 * @param string	$pKeyWordStr :キーワード文字列
 * @param bool		$pWhere : 文字列の先頭にWHEREを付けるかどうか。
 *
 * @return string	WHERE句の文字列を返す。
 * 					ex. (ColumnX LIKE ?) AND (ColumnX LIKE ?) AND (ColumnX LIKE ?)
 * @link https://www.softel.co.jp/blogs/tech/archives/5537 (2021/07確認)
 */
function MakeWherePhraseWithKeywords(&$pParams, $pKeyWordStr, $pWhere = false)
{
	$pParams = array();
	$where_array = array();
	if (empty($pKeyWordStr)) return ("");		//キーワードがなければ返る

	//キーワード前後の空白を取り除く
	$pKeyWordStr = preg_replace('/\A[\p{C}\p{Z}]++|[\p{C}\p{Z}]++\z/u', '', $pKeyWordStr);	//マルチバイト(全角スペース等)対応のtrim処理 https://qiita.com/fallout/items/a13cebb07015d421fde3
	//単語に分解
	$keyword_str = str_replace(array("　", "\t"), " ", $pKeyWordStr);
	$pParams = preg_split("/[ ]+/", $keyword_str);
	foreach ($pParams as $key => $val) {
		$pParams[$key] = '%' . $val . '%';		//WHERE句ではなくパラメータのほうに % を付ける
		$where_array[] = '(ColumnX LIKE ?)';
	}

	$where = $pWhere ? "WHERE " : "";
	$where_str = $where . implode(' AND ', $where_array) . " ";

	return ($where_str);
}
//---------------------------------------------------------------------------
/**
 * INSERT句で使う文字列を作る
 *
 * @param array		$pColumn : カラム名配列

 * @return string	INSERT句で使う文字列
 * ex." (`ユーザーID`,`所属`,`氏名`,`編集テーブル`,`操作ID`,`操作`,`結果`,`関数`) VALUES (?,?,?,?,?,?,?,?) "
 *
 * 使い方
 * $column = array('colum1','colum2','colum3','colum4');
 * $params = array('param1','param2','param3','param4');
 * $insert_str = MakeInsertStr($column);
 * $sql = "INSERT INTO `test_table` " . $insert_str;
 * $stmt = $pdo->prepare($sql);
 * $stmt->execute($params);
 */
function MakeInsertStr($pColumn)
{
	$str = " (`" . implode("`,`", $pColumn) . "`) VALUES (" . implode(',', array_fill(0, count($pColumn), '?')) . " ) ";
	return ($str);
}
//---------------------------------------------------------------------------
/**
 * 未定義のコマンド、$_REQUEST['Table']に不正の文字が入っていた
 *
 * @param string	$pMsg : ログに記録するメッセージ
 */
function UnExpected($pMsg = "未定義コマンド")
{
	//---------------処理部---------------
	//クッキーがない時のダミー
	if (!isset($_COOKIE[kUserID]))   $_COOKIE[kUserID]   = "No UserID";
	if (!isset($_COOKIE[kSection]))  $_COOKIE[kSection]  = "No Section";
	if (!isset($_COOKIE[kUserName])) $_COOKIE[kUserName] = "No UserName";
	if (!isset($_REQUEST['Cmd']))    $_REQUEST['Cmd']    = "No Cmd";

	$id = WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], '', '', $_REQUEST['Cmd'], $pMsg);
	//---------------表示部---------------
	include("prog/html01.inc");
	print <<< kUnExpected
<header>
パラメータエラー
</header>

<section>
	<div class="error msgbox">
	パラメータが不正です。(ログID:{$id})
	</div>
</section>

kUnExpected;
	include("prog/html11.inc");
	exit();
}
//---------------------------------------------------------------------------
/**
 * 変数の情報を出力し継続する
 *
 * @param mixed $p : 表示するデータ　可変長引数
 *
 * @usage 	ydebug(__FILE__, __LINE__, 表示したい変数（複数）);	__FILE__, __LINE__を入れておくとどこで使ったかがわかる
 */
function ydebug(...$p)
{
	echo "<pre>\n";
	print_r($p);
	echo "</pre>\n";
}
//---------------------------------------------------------------------------
/**
 * 変数の情報を出力し終了する
 *
 * @param mixed $p : 表示するデータ　可変長引数
 *
 * @usage 	zdebug(__FILE__, __LINE__, 表示したい変数（複数）);	__FILE__, __LINE__を入れておくとどこで使ったかがわかる
 */
function zdebug(...$p)
{
	ydebug(...$p);

	exit();
}
//---------------------------------------------------------------------------
