<?php
//---------------------------------------------------------------------------
//Charactor set is UTF-8
//漢字コードはＵＴＦ－８
//---------------------------------------------------------------------------
//ログイン関係
//---------------------------------------------------------------------------
/**
 * ログイン画面出力
 *
 * @param string $pMsg : ログイン失敗時のメッセージ
 */
function LoginScreen($pMsg = '')
{
	//---------------表示部---------------
	include("prog/html01.inc");

	print <<< kLoginScreen
<header>
ユーザー認証
</header>

<section>
	<form id="LoginForm" action="index.php" method="post">
		<input type="hidden" name="Cmd" value="ユーザー認証">
		<table>
			<tbody>
				<tr>
					<td>ユーザーID</td>
					<td><input type="text" name="UserID" required></td>
				</tr>
				<tr>
					<td>パスフレーズ</td>
					<td><input type="password" name="Pass"></td>
				</tr>
				<tr>
					<td colspan="2" class="right">
					<input type="reset" value="リセット">
					<input type="submit" id="Login" value="ログイン"></td>
				</tr>
			</tbody>
		</table>
	</form>
</section>

kLoginScreen;

	if (isset($pMsg)) {
		echo $pMsg;
	}
	include("prog/html11.inc");
}
//---------------------------------------------------------------------------
/**
 *
 * ユーザー認証
 * ユーザー、パスワードが違えばログイン画面を出して終了
 * 認証されれば、クッキーをセット
 *
 * @param string $_REQUEST['UserID'] : ユーザーID
 * @param string $_REQUEST['Pass'] : パスフレーズ
 */
function Login()
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	$login_err_msg   = "<div class=\"error msgbox\">ユーザーIDまたはパスフレーズが違います。</div>\n";
	$change_pass_msg = "<div class=\"error msgbox\">パスフレーズが設定されていません。パスフレーズを設定してください。</div>\n";

	if (!isset($_REQUEST['UserID'])) {
		WriteLog('---', '', '', '', '', $_REQUEST['Cmd'], 'UserIDの送信なし');
		LoginScreen($login_err_msg);
		exit();
	}

	try {
		$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

		//アカウント検査
		$sql = "SELECT COUNT(*) AS CNT FROM `" . kUSER_TABLE . "` WHERE `ユーザーID` = :id ";
		$stmt = $pdo->prepare($sql);
		$stmt->bindValue(':id', $_REQUEST['UserID'], PDO::PARAM_STR);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row['CNT'] == 0) {
			WriteLog($_REQUEST['UserID'], '', '', kUSER_TABLE, '', $_REQUEST['Cmd'], ' UserIDの登録なし');
			LoginScreen($login_err_msg);
			exit();
		}

		//パスフレーズ検査
		$sql = "SELECT * FROM `" . kUSER_TABLE . "` WHERE `ユーザーID` = :id ";
		$stmt = $pdo->prepare($sql);
		$stmt->bindValue(':id', $_REQUEST['UserID'], PDO::PARAM_STR);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if (empty($row['パスフレーズ'])) {	//パスフレーズが登録されていない場合は、パスフレーズの変更画面 !isset , is_null ではうまくいかなかった
			WriteLog($_REQUEST['UserID'], $row['所属'], $row['氏名'], kUSER_TABLE, $row['ユーザーID'], $_REQUEST['Cmd'], ' パスフレーズ未登録');
			ChangePassScreen($change_pass_msg);				//パスフレーズ変更画面
			exit();
		}

		if (!password_verify($_REQUEST['Pass'], $row['パスフレーズ'])) {	//パスフレーズが一致しない
			WriteLog($_REQUEST['UserID'], $row['所属'], $row['氏名'], kUSER_TABLE, $row['ユーザーID'], $_REQUEST['Cmd'], 'パスフレーズ不一致 ' . $_REQUEST['Pass']);
			LoginScreen($login_err_msg);
			exit();
		}

		//ログインに成功したのでクッキーをセットする
		if (empty($row['所属'])) $row['所属'] = '-';
		if (empty($row['管理フラグ'])) $row['管理フラグ'] = '一般';
		$exp = time() + 60 * 60 * kCookieExp;
		$tm  = date("Y/m/d H:i:s");
		setcookie(kLogin,     $tm, $exp);									//ログインクッキー
		setcookie(kUserID,    $row['ユーザーID'], $exp);
		setcookie(kSection,   $row['所属'], $exp);
		setcookie(kUserName,  $row['氏名'], $exp);
		setcookie(kUserLevel, $row['管理フラグ'], $exp);

		$_COOKIE[kLogin]     = $tm;											//Login()に続いてテーブルを表示するため
		$_COOKIE[kUserID]    = $row['ユーザーID'];
		$_COOKIE[kSection]   = $row['所属'];
		$_COOKIE[kUserName]  = $row['氏名'];
		$_COOKIE[kUserLevel] = $row['管理フラグ'];
		WriteLog($row['ユーザーID'], $row['所属'], $row['氏名'], kUSER_TABLE, $row['ユーザーID'], $_REQUEST['Cmd'], 'ログイン成功');
	} catch (PDOException $e) {
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$id = WriteLog($_REQUEST['UserID'], '', '', kUSER_TABLE, '', $_REQUEST['Cmd'], $err);
		$msg = "<div class=\"error msgbox\">データベースへの接続に失敗しました。(ログID:{$id})</div>\n";
		LoginScreen($msg);
		exit();
	}
}
//---------------------------------------------------------------------------
/**
 * パスフレーズ変更画面出力
 *
 * @param string $_REQUEST['UserID'] : ユーザーID
 * @param string $pMsg : パスフレーズ変更失敗時のメッセージ
 */
function ChangePassScreen($pMsg = '')
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	try {
		$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

		$sql = "SELECT * FROM `" . kUSER_TABLE . "` WHERE `ユーザーID` = :id ";
		$stmt = $pdo->prepare($sql);
		$stmt->bindValue(':id', $_REQUEST['UserID'], PDO::PARAM_STR);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
	} catch (PDOException $e) {
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$id = WriteLog($_REQUEST['UserID'], '', '', kUSER_TABLE, '', 'パスフレーズ変更画面', $err);
		$pMsg = $pMsg . "<div class=\"error msgbox\">データベースへの接続に失敗しました。(ログID:{$id})</div>\n";
	}
	$userid  = htmlspecialchars($_REQUEST['UserID'], ENT_QUOTES, 'UTF-8');
	$usrname = htmlspecialchars($row['氏名'], ENT_QUOTES, 'UTF-8');
	//---------------表示部---------------
	include("prog/html01.inc");
	print <<< kRstPass
<header>
パスフレーズ変更
</header>

<section>
	<form id="ChgPassForm" action="index.php" method="post">
		<input type="hidden" name="Cmd" value="パス変更">
		<input type="hidden" name="UserID" value="{$userid}">
		<table>
			<tbody>
				<tr>
					<td>ユーザーID／ユーザー名</td>
					<td>{$userid}／{$usrname}</td>
				</tr>
				<tr>
					<td>現在のパスフレーズ</td>
					<td><input type="password" name="CurPass" maxlength="72"></td>
				</tr>
				<tr>
					<td>新しいパスフレーズ</td>
					<td><input type="password" name="NewPass" required maxlength="72"></td>
				</tr>
				<tr>
					<td colspan="2" class="right">
					<input type="button" value="戻る" onclick="history.go(-1)">
					<input type="reset"  value="リセット">
					<input type="submit" value="設定"></td>
				</tr>
			</tbody>
		</table>
	</form>

	<div class="msgbox message">
	<span class="small">（独法）情報処理推進機構（ＩＰＡ）</span>
	<a href="https://www.ipa.go.jp/security/keihatsu/munekyun-pw/slideshow/index.html" target="_blank" alt="パスワードーもっと強くキミを守りたい">パスワードーもっと強くキミを守りたい</a> <span class="small">(外部リンク。別ウインドウ)</span>
	</div>

</section>

kRstPass;

	if (isset($pMsg)) {
		echo $pMsg;
	}
	include("prog/html11.inc");
}
//---------------------------------------------------------------------------
/**
 * パスフレーズ変更
 * ユーザー、現在のパスフレーズが違えばログイン画面を出して終了
 *
 * @param string $_REQUEST['UserID'] : ユーザーID
 * @param string $_REQUEST['CurPass'] : 現在のパスフレーズ
 * @param string $_REQUEST['NewPass'] : 新しいパスフレーズ
 *
 * @return string 成功のメッセージ
 */
function ChangePass()
{
	global $gPDOOption, $gDsn;
	$err_msg = "<div class=\"error msgbox\">ユーザーIDまたは現在のパスフレーズが違います。</div>\n";
	//---------------処理部---------------
	if (!isset($_REQUEST['UserID']) || !isset($_REQUEST['NewPass'])) {
		ChangePassScreen($err_msg);	//パスフレーズ変更画面
		exit();
	}

	try {
		$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

		//ユーザー確認
		$sql = "SELECT COUNT(*) AS CNT FROM `" . kUSER_TABLE . "` WHERE `ユーザーID` = :id ";
		$stmt = $pdo->prepare($sql);
		$stmt->bindValue(':id', $_REQUEST['UserID'], PDO::PARAM_STR);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		if ($row['CNT'] == 0) {
			WriteLog($_REQUEST['UserID'], '', '', kUSER_TABLE, '', $_REQUEST['Cmd'], 'UserIDの登録なし');
			LoginScreen($err_msg);
			exit();
		}

		//パスフレーズ確認
		$sql = "SELECT * FROM `" . kUSER_TABLE . "` WHERE `ユーザーID` = :id ";
		$stmt = $pdo->prepare($sql);
		$stmt->bindValue(':id', $_REQUEST['UserID'], PDO::PARAM_STR);
		$stmt->execute();
		$row = $stmt->fetch(PDO::FETCH_ASSOC);
		$stmt->closeCursor();

		if (!empty($row['パスフレーズ'])) {
			if (!password_verify($_REQUEST['CurPass'], $row['パスフレーズ'])) {	//登録してあるパスフレーズが一致しない
				WriteLog($row['ユーザーID'], $row['所属'], $row['氏名'], kUSER_TABLE, $row['ユーザーID'], $_REQUEST['Cmd'], 'パスフレーズ不一致 ' . $_REQUEST['CurPass']);
				ChangePassScreen($err_msg);
				exit();
			}
		}

		//パスフレーズ変更
		$pdo->beginTransaction();
		$hash_pass = password_hash($_REQUEST['NewPass'], PASSWORD_DEFAULT);
		if (isset($_COOKIE[kUserName])) {
			$nm = $_COOKIE[kUserName] . "\n" . $row['編集者'];
		} else {
			$nm = $row['氏名'] . "\n" . $row['編集者'];
		}


		$update_str = "`パスフレーズ` = ? , `更新回数` = `更新回数` + 1 , `編集者` = ? ";
		$where_str  = "`ユーザーID` = ? ";
		$params = array($hash_pass, $nm, $row['ユーザーID']);
		$sql = "UPDATE `" . kUSER_TABLE . "` SET {$update_str} WHERE {$where_str} ";
		$stmt = $pdo->prepare($sql);
		$stmt->execute($params);
		$pdo->commit();
		WriteLog($row['ユーザーID'], $row['所属'], $row['氏名'], kUSER_TABLE, $row['ユーザーID'], $_REQUEST['Cmd'], "成功");
		$msg = "<div class=\"message msgbox\">パスフレーズを変更しました。再度ログインしてください。</div>\n";
		return ($msg);
	} catch (PDOException $e) {
		$pdo->rollBack();
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$id = WriteLog($_REQUEST['UserID'], '', '', kUSER_TABLE, $_REQUEST['UserID'], 'パスフレーズ変更', $err);
		$msg = "<div class=\"error msgbox\">データベースへの接続に失敗しました。(ログID:{$id})</div>\n";
		ChangePassScreen($msg);
		exit();
	}
}
//---------------------------------------------------------------------------
/**
 * ログアウト
 * クッキーを空にする
 */
function Logout()
{
	//---------------処理部---------------
	if ($_COOKIE[kUserLevel] == '管理者') {		//ログインクッキークリア
		setcookie(kTable,     '', time() - 60);
		setcookie(kSortCol,   '', time() - 60);
		setcookie(kSortOrder, '', time() - 60);
	}
	setcookie(kLogin,     '', time() - 60);
	setcookie(kUserID,    '', time() - 60);
	setcookie(kSection,   '', time() - 60);
	setcookie(kUserName,  '', time() - 60);
	setcookie(kUserLevel, '', time() - 60);
	WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], '', '', $_REQUEST['Cmd'], '');
}
//---------------------------------------------------------------------------
