<?php
//---------------------------------------------------------------------------
//Charactor set is UTF-8
//漢字コードはＵＴＦ－８
//---------------------------------------------------------------------------
//テーブル表示関係
//---------------------------------------------------------------------------
/**
 * 選択されたテーブルの内容を表示する
 *
 * @param string	$_REQUEST['Table'] : 選択しているテーブルの番号
 * @param string	$_REQUEST['KeyWords'] : 検索キーワード
 * @param integer	$_REQUEST['Page'] : 表示ページ
 * @param string	$_REQUEST['SortCol'] : ソートカラムの番号
 * @param string	$_REQUEST['SortOrder'] : ソート順(ASC , DESC)
 */
function ViewTable()
{
	global $gTables;
	//---------------処理部---------------
	$primary_key = '';		//プライマリーキーの名前
	$fields = array();		//カラム名一覧
	$params = array();		//$stmt->executeのパラメータ
	$row_count = 0;			//行数
	$msg = '';

	//一覧表を作る
	try {
		$primary_key = GetColumns($fields, $columns);				//カラムの情報を得る
		$subquery	= MakeSubQuery($fields);						//サブクエリー文作成
		$from		= MakeFromPhrase($subquery);					//FROM句作成
		$where		= MakeWherePhraseWithKeywords($params, $_REQUEST['KeyWords'], true);		//WHERE句作成
		$order		= MakeOrderPhase($fields, $primary_key);		//ORDER句作成
		$limit		= MakeLimitPhase();								//LIMIT句作成

		$row_count	= CountRows($from, $where, $params);			//テーブルの行を数える
		$thead		= "<tr>\n<th></th>\n<th>" . implode("</th>\n<th>", $fields) . "</th>\n</tr>\n";		//タイトル行
		$tbody		= MakeTableBody($from, $where, $order, $limit, $params, $primary_key); //表示するデータを作成する table body

		if (empty($_REQUEST['KeyWords'])) {							//キーワードがあれば検索と記録
			WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], '');
		} else {
			WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', "検索", $_REQUEST['KeyWords']);
		}
	} catch (PDOException $e) {
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$id = WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], $err);
		$msg = "<div class=\"error msgbox\">データベースへの接続に失敗しました。(ログID:{$id})</div>\n";
	}

	//---------------表示部---------------
	include("prog/html01.inc");
	printf("\n<script src=\"prog/asuna02.js\"></script>\n");
	printf("<header>\nテーブル表示・データ編集\n</header>\n");

	if ($msg == "") {
		PrintTitle($row_count);				//メニュー
		if (empty($primary_key)) {
			printf("\n<div class=\"notice msgbox\">テーブルにプライマリーキーが設定されていないので、編集等できません。</div>\n");
		}
		EditItemMenu($fields, $primary_key); //行操作メニュー
		//テーブルの表示
		printf("<table class=\"stripe\">\n<thead>\n%s</thead>\n\n", $thead);
		printf("<tbody>\n");
		foreach ($tbody as $val) {			//テーブルの中身
			echo $val;
		}
		printf("</tbody>\n</table>\n</form>\n</section>\n\n");
	} else {
		echo nl2br($msg, false);
	}

	LogoutButton();
	include("prog/html11.inc");
}
//---------------------------------------------------------------------------
/**
 * テーブル表示のメニューを作って出力する
 *
 * @param string	$_REQUEST['Table'] : 選択しているテーブル
 * @param integer	$_REQUEST['Page'] : 表示ページ
 * @param string	$_REQUEST['SortCol'] : ソートカラムの番号
 * @param string	$_REQUEST['SortOrder'] : ソート順(ASC , DESC)
 * @param string	$_REQUEST['KeyWords'] :キーワード文字列
 * @param string	$pRow_count : 行数
 */
function PrintTitle($pRow_count)
{
	global $gTables;
	//---------------処理部---------------
	//テーブル選択メニュー
	$table_options = array();
	foreach ($gTables as $key => $val) {
		if ($key == $_REQUEST['Table']) {
			$table_options[] = sprintf("<option value=\"%s\" selected>%s</option>", $key, $val);	//$gTablesはconfig.incの設定なのでサニタイズの必要はない
		} else {
			$table_options[] = sprintf("<option value=\"%s\">%s</option>", $key, $val);
		}
	}
	$tables = implode("\n", $table_options);
	//ページ選択メニュー
	$pages = MakePageSelect($pRow_count);

	$key_words = htmlspecialchars($_REQUEST['KeyWords'], ENT_QUOTES, 'UTF-8');
	if (($pRow_count % kView) != 0) {
		$page_num = (int)($pRow_count / kView + 1);
	} else {
		$page_num = (int)($pRow_count / kView);
	}
	$sortcol   = htmlspecialchars($_REQUEST['SortCol'], ENT_QUOTES, 'UTF-8');
	$sortorder = htmlspecialchars($_REQUEST['SortOrder'], ENT_QUOTES, 'UTF-8');
	//---------------表示部---------------
	print <<< kPrintTitle
<section class="cCommand">
	<form action="index.php" id="ViewTableForm" method="post" style="display:inline">
		<input type="hidden" name="Cmd"       value="テーブル表示">
		<input type="hidden" name="SortCol"   value="{$sortcol}">
		<input type="hidden" name="SortOrder" value="{$sortorder}">
		テーブル
		<select id="Table" name="Table">
		{$tables}
		</select>
		表示ページ
		<select id="Page" name="Page">
		{$pages}
		</select>
		キーワード
		<input type="search" name="KeyWords" value="{$key_words}">
		<label><input type="image" id="search" src="icons/search_l_b.ico" name="検索・表示">検索・表示</label>
	</form>
	<span id="info1" style="font-size:85%;">({$pRow_count}行({$page_num}ページ)のデータがあります)</span>
</section>

kPrintTitle;
}
//---------------------------------------------------------------------------
/**
 * サブクエリー文作成
 *
 * @param string[]	$pFields : カラム名の配列
 * @param string	$_REQUEST['KeyWords'] : 検索キーワード
 * @param string	$_REQUEST['Table'] : 選択しているテーブルの番号
 *
 * @return string	サブクエリ文字列
 */
function MakeSubQuery($pFields)
{
	global $gTables;
	//---------------処理部---------------
	if (empty($_REQUEST['KeyWords'])) return ('');

	$a = implode('` , `', $pFields);
	$subquery = " ( SELECT `" . $a . "` , CONCAT_WS(' ' , `" . $a . "` ) AS ColumnX \n"
		. "FROM `" . $gTables[$_REQUEST['Table']] . "` \n) AS TableX ";
	return ($subquery);
}
//---------------------------------------------------------------------------
/**
 * FROM句作成
 *
 * @param string	$pSubquery : サブクエリ文
 * @param string	$_REQUEST['Table'] : 選択しているテーブルの番号
 *
 * @return string	: FROM句
 */
function MakeFromPhrase($pSubquery)
{
	global $gTables;
	//---------------処理部---------------
	if ($pSubquery == '') {
		$from = "FROM `" . $gTables[$_REQUEST['Table']] . "` \n";
	} else {
		$from = "FROM " . $pSubquery . " \n";
	}
	return ($from);
}
//---------------------------------------------------------------------------
/**
 * WHERE句作成--使っていない
 *
 * @param string	$_REQUEST['KeyWords'] : 検索キーワード
 * @param string	$gKeyWordArray : 検索キーワード配列
 *
 * @return string	: WHERE句
 */
function MakeWherePhase()
{
	global $gKeyWordArray;
	//---------------処理部---------------
	if (empty($_REQUEST['KeyWords'])) return ('');

	$wheres   = array();
	foreach ($gKeyWordArray as $val) {	//キーワードの回数だけ繰り返す
		$wheres[] = '(ColumnX LIKE ?)';	//LIKE %?% とするとエラーが出る
		//@link http://none.3rin.net/php/php%20pdo%20prepare%E3%81%A7like%EF%BC%88%E9%83%A8%E5%88%86%E4%B8%80%E8%87%B4%EF%BC%89 (2021/07確認)
		// % はデータに付ける
	}
	$where = "WHERE ( " . implode(" AND ", $wheres) . " ) \n";	//検索は論理積
	return ($where);
}
//---------------------------------------------------------------------------
/**
 * ORDER句作成
 * $_REQUESTが設定されていなければクッキーを使う
 * もしソートカラムがなければ ID でソートする
 *
 * @param string	$_REQUEST['SortCol'] : ソートするカラムの番号
 * @param string	$_REQUEST['SortOrder'] : ソート順 ASC , DESC
 *
 * @param string[]	$pFields : カラム名配列
 * @param string	$pPrimarykeyName : プライマリーキー（ない場合もある）
 *
 * @return string	: ORDER句
 */
function MakeOrderPhase($pFields, $pPrimarykeyName)
{
	//---------------処理部---------------
	if (!empty($_REQUEST['SortCol'])) {
		$sort = $pFields[$_REQUEST['SortCol']];
	} else if (!empty($pPrimarykeyName)) {
		$sort = $pPrimarykeyName;
	} else {
		$sort = '';
	}

	if (!empty($_REQUEST['SortOrder'])) {
		if ($_REQUEST['SortOrder'] == 'ASC' || $_REQUEST['SortOrder'] == 'DESC') {		//ソート順は ASC か DESC に限られる
			$sortder = $_REQUEST['SortOrder'];
		} else {
			$sortder = 'DESC';
		}
	} else {
		$sortder = 'DESC';
	}

	if (empty($sort)) {
		$order = '';
	} else {
		$order = sprintf("ORDER BY `%s` %s ", $sort, $sortder);
	}
	return ($order);
}
//---------------------------------------------------------------------------
/**
 * LIMIT句作成
 *
 * @param integer	$_REQUEST['Page'] : 表示するページ
 * @param integer	kView : 一覧表示する件数
 *
 * @return string	: LIMIT句
 */
function MakeLimitPhase()
{
	//---------------処理部---------------
	if (!empty($_REQUEST['Page'])) {
		$page = intval($_REQUEST['Page']);
	} else {
		$page = 1;
	}

	$limit = sprintf("LIMIT %d , %d ", ($page - 1) * kView, kView);
	return ($limit);
}
//---------------------------------------------------------------------------
/**
 * 行操作メニュー
 *
 * @param string[]	$pFields : カラム名配列
 * @param string	$pPrimarykeyName : プライマリーキーの名前
 *
 */
function EditItemMenu($pFields, $pPrimarykeyName)
{
	$sort_ary = array("ASC" => "昇順", "DESC" => "降順");
	//---------------処理部---------------
	//ソートカラム選択メニュー
	$column_options = array();
	foreach ($pFields as $key => $val) {
		if ($key == $_REQUEST['SortCol']) {
			$column_options[] = sprintf("<option value=\"%s\" selected>%s</option>", $key, $val);
		} else {
			$column_options[] = sprintf("<option value=\"%s\">%s</option>", $key, $val);
		}
	}
	$column = implode("\n", $column_options);
	//ソート順
	$srt_options = array();
	foreach ($sort_ary as $key => $val) {
		if ($key == $_REQUEST['SortOrder']) {
			$srt_options[] = sprintf("<option value=\"%s\" selected>%s</option>", $key, $val);
		} else {
			$srt_options[] = sprintf("<option value=\"%s\">%s</option>", $key, $val);
		}
	}
	$srt =	implode("\n", $srt_options);

	$table     = htmlspecialchars($_REQUEST['Table'],    ENT_QUOTES, 'UTF-8');
	$page      = htmlspecialchars($_REQUEST['Page'],     ENT_QUOTES, 'UTF-8');
	$key_words = htmlspecialchars($_REQUEST['KeyWords'], ENT_QUOTES, 'UTF-8');

	//閲覧のみのユーザー
	if (isset($_COOKIE[kUserLevel]) && ($_COOKIE[kUserLevel] == '閲覧のみ')) {
		$enable_edit = <<< kEditItemMenu1
		<fieldset>
			<legend>データを</legend>
			<label><input type="image" id="view"   alt="表示" src="icons/file_css_b.ico" title="表示" name="表示">表示</label>
		</fieldset>

kEditItemMenu1;
	} else {
		$enable_edit = <<< kEditItemMenu1
		<fieldset>
			<legend>新規入力</legend>
			<label><input type="image" id="new_item" alt="新規入力" src="icons/file_new_b.ico" title="新規入力" name="新規">新規</label>
		</fieldset>
		<fieldset>
			<legend>選択したデータを</legend>
			<label><input type="image" id="view"   alt="表示" src="icons/file_css_b.ico" title="表示" name="表示">表示</label>
			<label><input type="image" id="edit"   alt="編集" src="icons/pen_bl_b.ico"   title="編集" name="編集">編集</label>
			<label><input type="image" id="copy"   alt="複製" src="icons/copy_b.ico"     title="複製" name="複製">複製</label>
			<label><input type="image" id="delete" alt="削除" src="icons/file_del_b.ico" title="削除" name="削除">削除</label>
		</fieldset>

kEditItemMenu1;
	}

	//管理者ユーザー
	if (isset($_COOKIE[kUserLevel]) && ($_COOKIE[kUserLevel] == '管理者')) {
		$enable_kanri = <<< kEditItemMenu3
		<fieldset>
			<legend>テーブルを</legend>
			<label><input type="image" id="truncate" alt="空にする" src="icons/file_stop_b.ico" title="空にする" name="空にする">空にする</label>
		</fieldset>

kEditItemMenu3;
	} else {
		$enable_kanri = "";
	}
	//---------------表示部---------------
	print <<< kEditItemMenu2
	<section>
		<form action="index.php" method="post" id="EditItemForm" style="display:inline">
			<input type="hidden" name="Cmd"            value="データ編集">
			<input type="hidden" name="Table"          value="{$table}">
			<input type="hidden" name="Page"           value="{$page}">
			<input type="hidden" name="KeyWords"       value="{$key_words}">
			<input type="hidden" name="PrimarykeyName" value="{$pPrimarykeyName}">
{$enable_edit}
		<fieldset>
			<legend>並び順</legend>
			<select name="SortCol">
			{$column}
			</select>
			<select name="SortOrder">
			{$srt}
			</select>
			<label><input type="image" id="sort" alt="並び替え" src="icons/reload_b.ico" title="並び替え" name="並び替え">並び替え</label>
		</fieldset>
		<fieldset>
			<legend>データ出力</legend>
			<label><input type="image" id="csv" alt="出力" src="icons/save_1_b.ico" title="出力" name="CSV出力">CSV出力</label>
		</fieldset>

{$enable_kanri}

kEditItemMenu2;
}
//---------------------------------------------------------------------------
/**
 * 表示ページのセレクトメニューを作る
 *
 * @param integer	$_REQUEST['Page'] : 表示ページ
 * @param integer	$pCount : テーブルの行数
 * @param integer	kView : 一覧表に表示する行数
 *
 * @return string	表示ページのセレクトメニュー
 */
function MakePageSelect($pCount)
{
	$e6 = array(1.0, 1.5, 2.2, 3.3, 4.7, 6.8);		//E6系列でページを作る
	//---------------処理部---------------
	if (empty($_REQUEST['Page'])) {
		$page = 1;
	} else {
		$page = intval($_REQUEST['Page']);
	}

	if (($pCount % kView) != 0) {
		$num = (int)($pCount / kView + 1);
	} else {
		$num = (int)($pCount / kView);
	}

	$pnum = array();								//optionに入れるページの値の配列
	for ($i = $page - 5; $i <= $page + 5; $i++) {	//見たいページ+-5は詳しく
		$pnum[] = $i;
	}

	for ($i = 1; $i < $num; $i = $i * 10) {			//そのほかの場所はE6系列で作成
		foreach ($e6 as $val) {
			$pnum[] = (int)($val * $i);
		}
	}
	$pnum[] = 1;									//最初と最後のページ
	$pnum[] = $num;

	$pval = array_unique($pnum);					//重複を削除
	sort($pval);									//値でソート

	$pages = array();								//option文字列
	foreach ($pval as $val) {
		if ($val <= 0) continue;
		if ($val > $num) continue;

		if ($page == $val) {
			$pages[] = sprintf("<option value=\"%d\" selected>%d</option>", $val, $val);
		} else {
			$pages[] = sprintf("<option value=\"%d\">%d</option>", $val, $val);
		}
	}
	$pageselect = implode("\n", $pages);
	return ($pageselect);
}
//---------------------------------------------------------------------------
/**
 * CSV出力
 *
 * @param string	$_REQUEST['Table'] : 選択しているテーブルの番号
 * @param string	$_REQUEST['KeyWords'] : 検索キーワード
 * @param string	$_REQUEST['SortCol'] : ソートカラムの番号
 * @param string	$_REQUEST['SortOrder'] : ソート順(ASC , DESC)
 */
function CSVOut()
{
	global $gTables;
	//---------------処理部---------------
	$primary_key = '';			//プライマリーキーの名前
	$limit = "";				//CSV出力は件数制限なし
	$fields = array();			//カラム名一覧
	$params = array();			//$stmt->executeのパラメータ
	$msg = '';

	//一覧表を作る
	try {
		$primary_key = GetColumns($fields, $columns);				//カラムの情報を得る
		$subquery	= MakeSubQuery($fields);						//サブクエリー文作成
		$from		= MakeFromPhrase($subquery);					//FROM句作成
		$where		= MakeWherePhraseWithKeywords($params, $_REQUEST['KeyWords'], true);		//WHERE句作成
		$order		= MakeOrderPhase($fields, $primary_key);					//ORDER句作成
		$limit		= "";	//MakeLimitPhase();						//LIMIT句作成  CSV出力は件数制限なし

		//出力するデータを作成する
		$thead		= "\"" . implode("\",\"", $fields) . "\"\n";	//タイトル行
		$tbody		= MakeTableBodyCSV($from, $where, $order, $limit, $params);

		if (empty($_REQUEST['KeyWords'])) {							//キーワードがあれば検索
			WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], '');
		} else {
			WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'] . " 検索", $_REQUEST['KeyWords']);
		}
	} catch (PDOException $e) {
		$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
		$id = WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], $err);
		$msg = "データベースへの接続に失敗しました。(ログID:{$id})\n";
	}
	//---------------表示部---------------
	//CSVファイルとして出力するためのヘッダ
	header("Content-Type: application/octet-stream");
	header("Content-Disposition: attachment; filename={$gTables[$_REQUEST['Table']]}.csv");

	ob_start();		//出力をバッファリング
	if ($msg == '') {
		echo $thead;
		foreach ($tbody as $val) {
			echo $val;
		}
	} else {
		echo $msg;
	}
	$output = ob_get_contents();	//一気にShift_JISに変換して出力--多分バッファーは充分ある
	ob_end_clean();
	echo mb_convert_encoding($output, kCSV_CharSet, "UTF-8");
}
//---------------------------------------------------------------------------
/**
 * テーブルの行数を数える
 *
 * @param string	$pFrom : FROM句
 * @param string	$pWhere : WHERE句
 * @param array		$pParams : $stmt->executeのパラメータ
 *
 * @return integer : テーブルの行数
 */
function CountRows($pFrom, $pWhere, $pParams)
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

	//テーブルの行を数える
	$sql = "SELECT COUNT(*) AS CNT " . $pFrom . $pWhere;
	$stmt = $pdo->prepare($sql);
	if (count($pParams) >= 1) {
		$stmt->execute($pParams);
	} else {
		$stmt->execute();
	}

	$row = $stmt->fetch(PDO::FETCH_ASSOC);
	return ($row['CNT']);
}
//---------------------------------------------------------------------------
/**
 * 一覧表の中身を作る
 *
 * @param string	$pFrom : FROM句
 * @param string	$pWhere : WHERE句
 * @param string	$pOrder : ORDER句
 * @param string	$pLimit : LIMIT句
 * @param array		$pParams : $stmt->executeのパラメータ
 * @param string	$pPrimarykeyName : プライマリーキーの名前
 *
 * @return string[] : テーブルの中身
 */
function MakeTableBody($pFrom, $pWhere, $pOrder, $pLimit, $pParams, $pPrimarykeyName)
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

	$sql = "SELECT * " . $pFrom . $pWhere . $pOrder . $pLimit;
	$stmt = $pdo->prepare($sql);
	if (count($pParams) >= 1) {
		$stmt->execute($pParams);
	} else {
		$stmt->execute();
	}

	$tbody = array();
	while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
		unset($row['ColumnX']);

		if (!empty($pPrimarykeyName)) {	//プライマリーキーがある＝編集可能
			$radio = sprintf("<input type=\"radio\" name=\"PrimarykeyVal\" value=\"%s\">", $row[$pPrimarykeyName]);
		} else {
			$radio = "";
		}

		foreach ($row as $key => $val) {
			$row[$key] = mb_strimwidth($val, 0, kStrLen, '...', 'UTF-8');	//長いデータを短くして表示 マルチバイト文字は、通常はシングルバイト文字の倍の幅となります。
			$row[$key] = htmlspecialchars($row[$key], ENT_QUOTES, 'UTF-8');
		}

		$tbody[] = "<tr>\n<td>" . $radio . "</td>\n<td>" . implode("</td>\n<td>", $row) . "</td>\n</tr>\n\n";
	}
	return ($tbody);
}
//---------------------------------------------------------------------------
/**
 * 一覧表の中身を作る CSV用
 *
 * @param string	$pFrom : FROM句
 * @param string	$pWhere : WHERE句
 * @param string	$pOrder : ORDER句
 * @param string	$pLimit : LIMIT句  CSV出力では使わない
 * @param array		$pParams : $stmt->executeのパラメータ
 * @param string	$pPrimarykeyName : プライマリーキーの名前
 *
 * @return string[] : テーブルの中身
 */
function MakeTableBodyCSV($pFrom, $pWhere, $pOrder, $pLimit, $pParams)
{
	global $gPDOOption, $gDsn;
	//---------------処理部---------------
	$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

	$sql = "SELECT * " . $pFrom . $pWhere . $pOrder . $pLimit;
	$stmt = $pdo->prepare($sql);
	if (count($pParams) >= 1) {
		$stmt->execute($pParams);
	} else {
		$stmt->execute();
	}

	$tbody = array();
	while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
		unset($row['ColumnX']);
		$tbody[] = "\"" . implode("\",\"", $row) . "\"\n";
	}
	return ($tbody);
}
//---------------------------------------------------------------------------
/**
 * 空にする確認ウインドウを作る
 *
 * @param string $_REQUEST['Table'] : 選択しているテーブルの番号
 */
function TruncateTable()
{
	global $gTables;
	//---------------処理部---------------
	$table = htmlspecialchars($gTables[$_REQUEST['Table']], ENT_QUOTES, 'UTF-8');
	WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', "テーブルを空にする表示", "");
	//---------------表示部---------------
	include("prog/html01.inc");

	if (isset($_COOKIE[kUserLevel]) && ($_COOKIE[kUserLevel] == '管理者')) {
		print <<< kTruncateTable1
	<script src="prog/asuna02.js"></script>
	<header>
		テーブルを空にする
	</header>

	<section>
		<form action="index.php" id="TruncateForm" method="post">
			<input type="hidden" name="Cmd" value="空にする実行">
			<input type="hidden" name="Table" value="{$_REQUEST['Table']}">
			<div class="alert msgbox">テーブル（{$table}）のデータをすべて削除します。
			<input type="button" value="閉じる" onclick="window.close();">
			<input type="button" id="TruncateButton" value="実行">
			</div>
		</form>
	</section>

kTruncateTable1;
	} else {
		print <<< kTruncateTable2
	<header>
		テーブルを空にする
	</header>

	<section>
		<div class="message msgbox">
			管理者でなければテーブルを空にできません。<input type="button" value="閉じる" onClick="window.close();">
		</div>
	</section>

kTruncateTable2;
	}

	include("prog/html11.inc");
}
//---------------------------------------------------------------------------
/**
 * テーブルを空にする
 * MySQL は TRUNCATE TABLE に関して、トランザクションを効かせることができない
 * SQLServer , PostgreSQL ならばトランザクションが効くらしい
 *
 * @param string $_REQUEST['Cmd'] : '空にする'
 * @param string $_REQUEST['Table'] : 選択しているテーブルの番号
 */
function TruncateTableEx()
{
	global $gPDOOption, $gDsn;
	global $gTables;
	//---------------処理部---------------
	if (isset($_COOKIE[kUserLevel]) && ($_COOKIE[kUserLevel] == '管理者')) {
		try {
			$pdo = new PDO($gDsn, asuna_USER, asuna_PASS, $gPDOOption);

			$sql = sprintf("TRUNCATE TABLE `%s` ", $gTables[$_REQUEST['Table']]);
			$stmt = $pdo->prepare($sql);
			$stmt->execute();
			WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], "成功");
			$msg = "<div class=\"message msgbox\">テーブルを空にしました。<input type=\"button\" value=\"閉じる\" onClick=\"window.close()\"></div>\n";
		} catch (PDOException $e) {
			$err = sprintf("%s\n(%s  %d)", $e->getMessage(), $e->getFile(), $e->getLine());
			$id = WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], $err);
			$msg = "<div class=\"error msgbox\">データベースへの接続に失敗しました。(ログID:{$id})<input type=\"button\" value=\"閉じる\" onClick=\"window.close()\"></div>\n";
		}
	} else {
		$msg = "<div class=\"message msgbox\">管理者でなければテーブルを空にできません。<input type=\"button\" value=\"閉じる\" onClick=\"window.close()\"></div>\n";
		WriteLog($_COOKIE[kUserID], $_COOKIE[kSection], $_COOKIE[kUserName], $gTables[$_REQUEST['Table']], '', $_REQUEST['Cmd'], "失敗(管理者でない)");
	}
	//---------------表示部---------------
	include("prog/html01.inc");
	printf("<script>window.opener.$('#search').click();</script>\n<header>\nテーブルを空にする\n</header>\n\n<section>\n\n");
	echo nl2br($msg, false);
	printf("\n</section>\n");
	include("prog/html11.inc");
}
//---------------------------------------------------------------------------
